from pathlib import Path
import numpy as np
import matplotlib.pyplot as plt
from skimage.io import imread
from skimage.feature import match_template, peak_local_max
from skimage.transform import rescale
from dataclasses import dataclass
from argparse import ArgumentParser

@dataclass
class Image:
    name: str
    data: np.ndarray

parser = ArgumentParser(description='Image reader')
parser.add_argument('-t', '--threshold',
                    type=float,
                    default=0.8,
                    help='Detection threshold for peak_local_max function.')
parser.add_argument('-r', '--resize-alphabet',
                    dest='ratio',
                    type=float,
                    default=1.0,
                    help='Ratio for resizing alphabet images.')

args = parser.parse_args()

def load_images_from_dir(path: str, rescaling: bool = False) -> list[Image]:
    files: list[Image] = []
    for entry in Path(path).iterdir():
        if entry.is_file():
            image_data: np.ndarray = imread(entry.resolve(), as_gray=True)
            if rescaling and args.ratio != 1.0:
                image_data = rescale(image_data, args.ratio, anti_aliasing=True)
            files.append(Image(name=entry.stem, data=image_data))
    return files


def plot_detection(text_image: Image, letters_positions: dict, detected_word: str):
    fig, ax = plt.subplots(figsize=(15, 5))
    ax.imshow(text_image.data, cmap="gray")
    ax.set_axis_off()
    ax.set_title(f"{detected_word}", fontsize=16, color="blue")
    for (x, y), letter in letters_positions.items():
        h, w = next(letter_obj.data.shape for letter_obj in letters if letter_obj.name == letter)
        rect = plt.Rectangle((x, y), w, h, edgecolor='lime', facecolor='none', linewidth=2)
        ax.add_patch(rect)
    plt.show()
letters: list[Image] = load_images_from_dir("letters", rescaling=True)
text_images: list[Image] = load_images_from_dir("images")
for text in text_images:
    letters_positions: dict[tuple[int, int], str] = {}
    for letter in letters:
        result: np.ndarray = match_template(text.data, letter.data)
        peaks: np.ndarray = peak_local_max(result, threshold_abs=args.threshold, min_distance=10)
        for y, x in peaks:
            letters_positions[(x, y)] = letter.name
    keys: list[tuple[int, int]] = sorted(letters_positions.keys(), key=lambda x: x[0])
    detected_word = ''.join([letters_positions[key] for key in keys])
    plot_detection(text, letters_positions, detected_word)
